/*******************************************************************************
*    ALMA - Atacama Large Millimiter Array
*    (c) European Southern Observatory, 2002
*    Copyright by ESO (in the framework of the ALMA collaboration)
*    and Cosylab 2002, All rights reserved
*
*    This library is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Lesser General Public
*    License as published by the Free Software Foundation; either
*    version 2.1 of the License, or (at your option) any later version.
*
*    This library is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Lesser General Public License for more details.
*
*    You should have received a copy of the GNU Lesser General Public
*    License along with this library; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
* 
*/


#include <acsCoursePSImpl.h>
#include <acscourseAlarmsC.h>

#include "ACSAlarmSystemInterfaceFactory.h"
#include "AlarmSystemInterface.h"
#include "FaultState.h"
#include "faultStateConstants.h"

using namespace acscomponent;
using acsalarm::AlarmSystemInterface;

PS::PS(const ACE_CString &name,maci::ContainerServices * containerServices) : 
    ACSComponentImpl(name, containerServices)
{
	ACS_TRACE("::PS::PS");
	m_conSvcs=containerServices;
     
}

PS::~PS()
{
}

void PS::initialize() throw (ACSErr::ACSbaseExImpl) {
	// Get the mount component
    mount = m_conSvcs->getComponent<acscourse::Mount>("ALARM_COURSE_MOUNT");
}

void PS::execute()  throw (ACSErr::ACSbaseExImpl) {
}

void PS::cleanUp() {
	m_conSvcs->releaseComponent("ALARM_COURSE_MOUNT");
}

void PS::aboutToAbort() {
}

void PS::cutOff() throw (CORBA::SystemException ) {
	// Simulate the failure of the three generator
	// For each generator failure an alarm must be published
	sendAlarm("PowerSupplies","ALARM_COURSE_PS",1,true);
	sleep(1);
	sendAlarm("PowerSupplies","ALARM_COURSE_PS",2,true);
	sleep(1);
	sendAlarm("PowerSupplies","ALARM_COURSE_PS",3,true);
	sleep(1);
	// Simulate the failure of the power in the Mount
	ACS_SHORT_LOG((LM_INFO,"Propagating failure to Mount"));
	mount->powerOff();
}

void PS::powerOn() throw (CORBA::SystemException ) {
	// The power has been restored
	// For each generator a termination alarm must be sent
	sendAlarm("PowerSupplies","ALARM_COURSE_PS",1,false);
	sleep(1);
	sendAlarm("PowerSupplies","ALARM_COURSE_PS",2,false);
	sleep(1);
	sendAlarm("PowerSupplies","ALARM_COURSE_PS",3,false);
	sleep(1);
	// Simulate the power ON status to the Mount
	ACS_SHORT_LOG((LM_INFO,"Notifying Mount that the power is on"));
	mount->powerOn();
}

void PS::sendAlarm(std::string family, std::string member, int code, bool active) {
		// create the AlarmSystemInterface
		auto_ptr<AlarmSystemInterface> alarmSource = ACSAlarmSystemInterfaceFactory::createSource("ALARM_SYSTEM_SOURCES");

		// create the FaultState
		auto_ptr<acsalarm::FaultState> fltstate = ACSAlarmSystemInterfaceFactory::createFaultState(family, member, code);

		// set the fault state's descriptor
		string stateString;
		if (active) 
		{
			stateString = faultState::ACTIVE_STRING;
		} else {
			stateString = faultState::TERMINATE_STRING;
		}
		fltstate->setDescriptor(stateString);
		
		// create a Timestamp and use it to configure the FaultState
		Timestamp * tstampPtr = new Timestamp();
		auto_ptr<Timestamp> tstampAutoPtr(tstampPtr);
		fltstate->setUserTimestamp(tstampAutoPtr);

		// create a Properties object and configure it, then assign to the FaultState
		Properties * propsPtr = new Properties();
		propsPtr->setProperty(faultState::ASI_PREFIX_PROPERTY_STRING, "prefix");
		propsPtr->setProperty(faultState::ASI_SUFFIX_PROPERTY_STRING, "suffix");
		propsPtr->setProperty("TEST_PROPERTY", "TEST_VALUE");
		auto_ptr<Properties> propsAutoPtr(propsPtr);
		fltstate->setUserProperties(propsAutoPtr);

		// push the FaultState using the AlarmSystemInterface previously created
		//acsalarm::FaultState stateToPush(*fltstate);
		alarmSource->push(*fltstate);
}

/* --------------- [ MACI DLL support functions ] -----------------*/

#include <maciACSComponentDefines.h>
MACI_DLL_SUPPORT_FUNCTIONS(PS)
/* ----------------------------------------------------------------*/

